@@ -35,70 +35,82 @@ module Network.TLS.Extra.Cipher
     , cipher_DHE_DSS_RC4_SHA1
     , cipher_DHE_RSA_AES128GCM_SHA256
     , cipher_ECDHE_RSA_AES128GCM_SHA256
+    , cipher_ECDHE_RSA_AES128CBC_SHA256
+    , cipher_ECDHE_RSA_AES128CBC_SHA
     ) where
 
 import qualified Data.ByteString as B
 
 import Network.TLS (Version(..))
 import Network.TLS.Cipher
-import qualified "cipher-rc4" Crypto.Cipher.RC4 as RC4
+import Data.Tuple (swap)
 
-import qualified Crypto.Hash.SHA256 as SHA256
-import qualified Crypto.Hash.SHA1 as SHA1
-import qualified Crypto.Hash.MD5 as MD5
-
-import qualified "cipher-aes" Crypto.Cipher.AES as AES
+import Crypto.Cipher.AES
+import qualified Crypto.Cipher.RC4 as RC4
 import Crypto.Cipher.TripleDES
-import Crypto.Cipher.Types (makeKey, makeIV, cipherInit, cbcEncrypt, cbcDecrypt)
-import qualified Crypto.Cipher.Types as T
+import Crypto.Cipher.Types hiding (Cipher, cipherName)
+import Crypto.Error
 
-aes_cbc_encrypt :: Key -> IV -> B.ByteString -> B.ByteString
-aes_cbc_encrypt key iv d = AES.encryptCBC (AES.initAES key) iv d
+takelast :: Int -> B.ByteString -> B.ByteString
+takelast i b = B.drop (B.length b - i) b
 
-aes_cbc_decrypt :: Key -> IV -> B.ByteString -> B.ByteString
-aes_cbc_decrypt key iv d = AES.decryptCBC (AES.initAES key) iv d
+aes128cbc :: BulkDirection -> BulkKey -> BulkBlock
+aes128cbc BulkEncrypt key =
+    let ctx = noFail (cipherInit key) :: AES128
+     in (\iv input -> let output = cbcEncrypt ctx (makeIV_ iv) input in (output, takelast 16 output))
+aes128cbc BulkDecrypt key =
+    let ctx = noFail (cipherInit key) :: AES128
+     in (\iv input -> let output = cbcDecrypt ctx (makeIV_ iv) input in (output, takelast 16 input))
 
-aes128_cbc_encrypt
-  , aes128_cbc_decrypt
-  , aes256_cbc_encrypt
-  , aes256_cbc_decrypt :: Key -> IV -> B.ByteString -> B.ByteString
-aes128_cbc_encrypt = aes_cbc_encrypt
-aes128_cbc_decrypt = aes_cbc_decrypt
-aes256_cbc_encrypt = aes_cbc_encrypt
-aes256_cbc_decrypt = aes_cbc_decrypt
+aes256cbc :: BulkDirection -> BulkKey -> BulkBlock
+aes256cbc BulkEncrypt key =
+    let ctx = noFail (cipherInit key) :: AES256
+     in (\iv input -> let output = cbcEncrypt ctx (makeIV_ iv) input in (output, takelast 16 output))
+aes256cbc BulkDecrypt key =
+    let ctx = noFail (cipherInit key) :: AES256
+     in (\iv input -> let output = cbcDecrypt ctx (makeIV_ iv) input in (output, takelast 16 input))
 
-aes128_gcm_encrypt, aes128_gcm_decrypt :: Key -> Nonce -> B.ByteString -> AdditionalData -> (B.ByteString, T.AuthTag)
-aes128_gcm_encrypt key nonce d ad = AES.encryptGCM (AES.initAES key) nonce ad d
-aes128_gcm_decrypt key nonce d ad = AES.decryptGCM (AES.initAES key) nonce ad d
+aes128gcm :: BulkDirection -> BulkKey -> BulkAEAD
+aes128gcm BulkEncrypt key =
+    let ctx = noFail (cipherInit key) :: AES128
+     in (\nonce d ad ->
+            let aeadIni = noFail (aeadInit AEAD_GCM ctx nonce)
+             in swap $ aeadSimpleEncrypt aeadIni ad d 16)
+aes128gcm BulkDecrypt key =
+    let ctx = noFail (cipherInit key) :: AES128
+     in (\nonce d ad ->
+            let aeadIni = noFail (aeadInit AEAD_GCM ctx nonce)
+             in simpleDecrypt aeadIni ad d)
+  where
+    simpleDecrypt aeadIni header input = (output, tag)
+      where
+            aead                = aeadAppendHeader aeadIni header
+            (output, aeadFinal) = aeadDecrypt aead input
+            tag                 = aeadFinalize aeadFinal 16
 
-tripledes_ede_cbc_encrypt :: Key -> IV -> B.ByteString -> B.ByteString
-tripledes_ede_cbc_encrypt key iv bs =
-    cbcEncrypt (cipherInit $ tripledes_key key) (tripledes_iv iv) bs
+noFail :: CryptoFailable a -> a
+noFail = throwCryptoError
 
-tripledes_ede_cbc_decrypt :: Key -> IV -> B.ByteString -> B.ByteString
-tripledes_ede_cbc_decrypt key iv bs =
-    cbcDecrypt (cipherInit $ tripledes_key key) (tripledes_iv iv) bs
+makeIV_ :: BlockCipher a => B.ByteString -> IV a
+makeIV_ = maybe (error "makeIV_") id . makeIV
 
-tripledes_key :: Key -> T.Key DES_EDE3
-tripledes_key key = either (\ke -> error ("tripledes cipher key internal error: " ++ show ke)) id $ makeKey key
+tripledes_ede :: BulkDirection -> BulkKey -> BulkBlock
+tripledes_ede BulkEncrypt key =
+    let ctx = noFail $ cipherInit key
+     in (\iv input -> let output = cbcEncrypt ctx (tripledes_iv iv) input in (output, takelast 16 output))
+tripledes_ede BulkDecrypt key =
+    let ctx = noFail $ cipherInit key
+     in (\iv input -> let output = cbcDecrypt ctx (tripledes_iv iv) input in (output, takelast 16 input))
 
-tripledes_iv :: IV -> T.IV DES_EDE3
+tripledes_iv :: BulkIV -> IV DES_EDE3
 tripledes_iv iv = maybe (error "tripledes cipher iv internal error") id $ makeIV iv
 
-toIV :: RC4.Ctx -> IV
-toIV (RC4.Ctx ctx) = ctx
-
-toCtx :: IV -> RC4.Ctx
-toCtx iv = RC4.Ctx iv
-
-initF_rc4 :: Key -> IV
-initF_rc4 key     = toIV $ RC4.initCtx key
-
-encryptF_rc4 :: IV -> B.ByteString -> (B.ByteString, IV)
-encryptF_rc4 iv d = (\(ctx, e) -> (e, toIV ctx)) $ RC4.combine (toCtx iv) d
-
-decryptF_rc4 :: IV -> B.ByteString -> (B.ByteString, IV)
-decryptF_rc4 iv e = (\(ctx, d) -> (d, toIV ctx)) $ RC4.combine (toCtx iv) e
+rc4 :: BulkDirection -> BulkKey -> BulkStream
+rc4 _ bulkKey = BulkStream (combineRC4 $ RC4.initialize bulkKey)
+  where
+    combineRC4 ctx input =
+        let (ctx', output) = RC4.combine ctx input
+         in (output, BulkStream (combineRC4 ctx'))
 
 
 -- | all encrypted ciphers supported ordered from strong to weak.
@@ -141,68 +153,62 @@ bulk_null = Bulk
     { bulkName         = "null"
     , bulkKeySize      = 0
     , bulkIVSize       = 0
+    , bulkExplicitIV   = 0
+    , bulkAuthTagLen   = 0
     , bulkBlockSize    = 0
-    , bulkF            = BulkStreamF (const B.empty) streamId streamId
+    , bulkF            = BulkStreamF passThrough
     }
-    where streamId = \iv b -> (b,iv)
+  where
+    passThrough _ _ = BulkStream go where go inp = (inp, BulkStream go)
 
 bulk_rc4 = Bulk
     { bulkName         = "RC4-128"
     , bulkKeySize      = 16
     , bulkIVSize       = 0
+    , bulkExplicitIV   = 0
+    , bulkAuthTagLen   = 0
     , bulkBlockSize    = 0
-    , bulkF            = BulkStreamF initF_rc4 encryptF_rc4 decryptF_rc4
+    , bulkF            = BulkStreamF rc4
     }
 
 bulk_aes128 = Bulk
     { bulkName         = "AES128"
     , bulkKeySize      = 16
     , bulkIVSize       = 16
+    , bulkExplicitIV   = 0
+    , bulkAuthTagLen   = 0
     , bulkBlockSize    = 16
-    , bulkF            = BulkBlockF aes128_cbc_encrypt aes128_cbc_decrypt
+    , bulkF            = BulkBlockF aes128cbc
     }
 
 bulk_aes128gcm = Bulk
     { bulkName         = "AES128GCM"
     , bulkKeySize      = 16 -- RFC 5116 Sec 5.1: K_LEN
     , bulkIVSize       = 4  -- RFC 5288 GCMNonce.salt, fixed_iv_length
+    , bulkExplicitIV   = 8
+    , bulkAuthTagLen   = 16
     , bulkBlockSize    = 0  -- dummy, not used
-    , bulkF            = BulkAeadF aes128_gcm_encrypt aes128_gcm_decrypt
+    , bulkF            = BulkAeadF aes128gcm
     }
 
 bulk_aes256 = Bulk
     { bulkName         = "AES256"
     , bulkKeySize      = 32
     , bulkIVSize       = 16
+    , bulkExplicitIV   = 0
+    , bulkAuthTagLen   = 0
     , bulkBlockSize    = 16
-    , bulkF            = BulkBlockF aes256_cbc_encrypt aes256_cbc_decrypt
+    , bulkF            = BulkBlockF aes256cbc
     }
 
 bulk_tripledes_ede = Bulk
     { bulkName      = "3DES-EDE-CBC"
     , bulkKeySize   = 24
     , bulkIVSize    = 8
+    , bulkExplicitIV = 0
+    , bulkAuthTagLen = 0
     , bulkBlockSize = 8
-    , bulkF         = BulkBlockF tripledes_ede_cbc_encrypt tripledes_ede_cbc_decrypt
-    }
-
-hash_md5, hash_sha1, hash_sha256 :: Hash
-hash_md5 = Hash
-    { hashName = "MD5"
-    , hashSize = 16
-    , hashF    = MD5.hash
-    }
-
-hash_sha1 = Hash
-    { hashName = "SHA1"
-    , hashSize = 20
-    , hashF    = SHA1.hash
-    }
-
-hash_sha256 = Hash
-    { hashName = "SHA256"
-    , hashSize = 32
-    , hashF    = SHA256.hash
+    , bulkF         = BulkBlockF tripledes_ede
     }
 
 -- | unencrypted cipher using RSA for key exchange and MD5 for digest
@@ -211,7 +217,7 @@ cipher_null_MD5 = Cipher
     { cipherID           = 0x1
     , cipherName         = "RSA-null-MD5"
     , cipherBulk         = bulk_null
-    , cipherHash         = hash_md5
+    , cipherHash         = MD5
     , cipherKeyExchange  = CipherKeyExchange_RSA
     , cipherMinVer       = Nothing
     }
@@ -222,7 +228,7 @@ cipher_null_SHA1 = Cipher
     { cipherID           = 0x2
     , cipherName         = "RSA-null-SHA1"
     , cipherBulk         = bulk_null
-    , cipherHash         = hash_sha1
+    , cipherHash         = SHA1
     , cipherKeyExchange  = CipherKeyExchange_RSA
     , cipherMinVer       = Nothing
     }
@@ -233,7 +239,7 @@ cipher_RC4_128_MD5 = Cipher
     { cipherID           = 0x04
     , cipherName         = "RSA-rc4-128-md5"
     , cipherBulk         = bulk_rc4
-    , cipherHash         = hash_md5
+    , cipherHash         = MD5
     , cipherKeyExchange  = CipherKeyExchange_RSA
     , cipherMinVer       = Nothing
     }
@@ -244,7 +250,7 @@ cipher_RC4_128_SHA1 = Cipher
     { cipherID           = 0x05
     , cipherName         = "RSA-rc4-128-sha1"
     , cipherBulk         = bulk_rc4
-    , cipherHash         = hash_sha1
+    , cipherHash         = SHA1
     , cipherKeyExchange  = CipherKeyExchange_RSA
     , cipherMinVer       = Nothing
     }
@@ -255,7 +261,7 @@ cipher_AES128_SHA1 = Cipher
     { cipherID           = 0x2f
     , cipherName         = "RSA-aes128-sha1"
     , cipherBulk         = bulk_aes128
-    , cipherHash         = hash_sha1
+    , cipherHash         = SHA1
     , cipherKeyExchange  = CipherKeyExchange_RSA
     , cipherMinVer       = Just SSL3
     }
@@ -266,7 +272,7 @@ cipher_AES256_SHA1 = Cipher
     { cipherID           = 0x35
     , cipherName         = "RSA-aes256-sha1"
     , cipherBulk         = bulk_aes256
-    , cipherHash         = hash_sha1
+    , cipherHash         = SHA1
     , cipherKeyExchange  = CipherKeyExchange_RSA
     , cipherMinVer       = Just SSL3
     }
@@ -277,7 +283,7 @@ cipher_AES128_SHA256 = Cipher
     { cipherID           = 0x3c
     , cipherName         = "RSA-aes128-sha256"
     , cipherBulk         = bulk_aes128
-    , cipherHash         = hash_sha256
+    , cipherHash         = SHA256
     , cipherKeyExchange  = CipherKeyExchange_RSA
     , cipherMinVer       = Just TLS12
     }
@@ -288,7 +294,7 @@ cipher_AES256_SHA256 = Cipher
     { cipherID           = 0x3d
     , cipherName         = "RSA-aes256-sha256"
     , cipherBulk         = bulk_aes256
-    , cipherHash         = hash_sha256
+    , cipherHash         = SHA256
     , cipherKeyExchange  = CipherKeyExchange_RSA
     , cipherMinVer       = Just TLS12
     }
@@ -299,7 +305,7 @@ cipher_DHE_RSA_AES128_SHA1 = Cipher
     { cipherID           = 0x33
     , cipherName         = "DHE-RSA-AES128-SHA1"
     , cipherBulk         = bulk_aes128
-    , cipherHash         = hash_sha1
+    , cipherHash         = SHA1
     , cipherKeyExchange  = CipherKeyExchange_DHE_RSA
     , cipherMinVer       = Nothing
     }
@@ -318,7 +324,7 @@ cipher_DHE_DSS_AES128_SHA1 = Cipher
     { cipherID           = 0x32
     , cipherName         = "DHE-DSA-AES128-SHA1"
     , cipherBulk         = bulk_aes128
-    , cipherHash         = hash_sha1
+    , cipherHash         = SHA1
     , cipherKeyExchange  = CipherKeyExchange_DHE_DSS
     , cipherMinVer       = Nothing
     }
@@ -342,7 +348,7 @@ cipher_DHE_RSA_AES128_SHA256 :: Cipher
 cipher_DHE_RSA_AES128_SHA256 = cipher_DHE_RSA_AES128_SHA1
     { cipherID           = 0x67
     , cipherName         = "DHE-RSA-AES128-SHA256"
-    , cipherHash         = hash_sha256
+    , cipherHash         = SHA256
     , cipherMinVer       = Just TLS12
     }
 
@@ -359,7 +365,7 @@ cipher_RSA_3DES_EDE_CBC_SHA1 = Cipher
     { cipherID           = 0x0a
     , cipherName         = "RSA-3DES-EDE-CBC-SHA1"
     , cipherBulk         = bulk_tripledes_ede
-    , cipherHash         = hash_sha1
+    , cipherHash         = SHA1
     , cipherKeyExchange  = CipherKeyExchange_RSA
     , cipherMinVer       = Nothing
     }
@@ -369,7 +375,7 @@ cipher_DHE_RSA_AES128GCM_SHA256 = Cipher
     { cipherID           = 0x9e
     , cipherName         = "DHE-RSA-AES128GCM-SHA256"
     , cipherBulk         = bulk_aes128gcm
-    , cipherHash         = hash_sha256
+    , cipherHash         = SHA256
     , cipherKeyExchange  = CipherKeyExchange_DHE_RSA
     , cipherMinVer       = Just TLS12 -- RFC 5288 Sec 4
     }
@@ -379,7 +385,28 @@ cipher_ECDHE_RSA_AES128GCM_SHA256 = Cipher
     { cipherID           = 0xc02f
     , cipherName         = "ECDHE-RSA-AES128GCM-SHA256"
     , cipherBulk         = bulk_aes128gcm
-    , cipherHash         = hash_sha256
+    , cipherHash         = SHA256
+    , cipherKeyExchange  = CipherKeyExchange_ECDHE_RSA
+    , cipherMinVer       = Just TLS12 -- RFC 5288 Sec 4
+    }
+
+cipher_ECDHE_RSA_AES128CBC_SHA :: Cipher
+cipher_ECDHE_RSA_AES128CBC_SHA = Cipher
+    { cipherID           = 0xc013
+    , cipherName         = "ECDHE-RSA-AES128CBC-SHA"
+    , cipherBulk         = bulk_aes128
+    , cipherHash         = SHA1
+    , cipherKeyExchange  = CipherKeyExchange_ECDHE_RSA
+    , cipherMinVer       = Just TLS10
+    }
+
+--TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA256 
+cipher_ECDHE_RSA_AES128CBC_SHA256 :: Cipher
+cipher_ECDHE_RSA_AES128CBC_SHA256 = Cipher
+    { cipherID           = 0xc027
+    , cipherName         = "ECDHE-RSA-AES128CBC-SHA"
+    , cipherBulk         = bulk_aes128
+    , cipherHash         = SHA256
     , cipherKeyExchange  = CipherKeyExchange_ECDHE_RSA
     , cipherMinVer       = Just TLS12 -- RFC 5288 Sec 4
     }
